@@ -1,4 +1,3 @@
-//This is exactly the same as IF, hence the same implementation
 SEXP C_CMIM(SEXP X,SEXP Y,SEXP K,SEXP Threads){
  int n,k,m,ny,*y,*nx,**x,nt;
  struct ht **hta;
@@ -6,6 +5,7 @@ SEXP C_CMIM(SEXP X,SEXP Y,SEXP K,SEXP Threads){
 
  double bs=0.,*ms=(double*)R_alloc(sizeof(double),m);
  int bi=0,*cYc,*ctmp;
+ bi=0;
  initialMiScan(hta,n,m,y,ny,x,nx,&cYc,&ctmp,ms,&bs,&bi,nt);
  if(bs==0) return(makeAns(0,NULL,NULL));
  
@@ -15,7 +15,10 @@ SEXP C_CMIM(SEXP X,SEXP Y,SEXP K,SEXP Threads){
   *cW=(int*)R_alloc(sizeof(int),n*k),
   *wy=(int*)R_alloc(sizeof(int),n*k),
   *nwy=(int*)R_alloc(sizeof(int),k),
-  *cWY=(int*)R_alloc(sizeof(int),n*k);
+  *cWY=(int*)R_alloc(sizeof(int),n*k),
+  *cXWc=(int*)R_alloc(sizeof(int),n*nt),
+  *xwc=(int*)R_alloc(sizeof(int),n*nt),
+  *wy2w=(int*)R_alloc(sizeof(int),n*k);
  w[0]=x[bi]; nw[0]=nx[bi]; x[bi]=NULL;
  for(int e=0;e<m;e++) lk[e]=0;
  double *score; int *idx,ke=k;
@@ -32,10 +35,13 @@ SEXP C_CMIM(SEXP X,SEXP Y,SEXP K,SEXP Threads){
    bs=-INFINITY;
    int off=n*(e-1);
    nwy[e-1]=fillHt(hta[0],n,nw[e-1],w[e-1],ny,y,wy+off,cW+off,NULL,1);
-   for(int ee=0;ee<nwy[e-1];ee++) (cWY+off)[ee]=hta[0]->cnt[ee].c;
+   transHt(hta[0],wy2w+off,NULL);
+   mixCountsHt(hta[0],cWY+off);
+   //for(int ee=0;ee<nwy[e-1];ee++) (cWY+off)[ee]=hta[0]->cnt[ee].c;
   }
   double tbs=-INFINITY;
   int tbi=-1,tn=omp_get_thread_num(),*cX=cYc+(tn*n);
+  int *xw=xwc+tn*n,*cXW=cXWc+tn*n;
   struct ht *ht=hta[tn];
   #pragma omp for schedule(dynamic)
   for(int ee=0;ee<m;ee++){
@@ -45,12 +51,10 @@ SEXP C_CMIM(SEXP X,SEXP Y,SEXP K,SEXP Threads){
    //Push forward towards e
    for(;lk[ee]<e;lk[ee]++){
     int ew=lk[ee],off=ew*n;
-    //WY x[ee] with wy making first part of cmi
-    fillHt(ht,n,nx[ee],x[ee],nwy[ew],wy+off,NULL,cX,NULL,0);
-    double cmi=miHt(ht,cX,cWY+off);
-    //WY x[ee] with w to make second part of CMI
-    fillHt(ht,n,nw[ew],w[ew],nx[ee],x[ee],NULL,NULL,NULL,0); 
-    cmi-=miHt(ht,cW+off,cX);
+    int nxw=fillHt(ht,n,nx[ee],x[ee],nw[ew],w[ew],xw,cX,NULL,1);
+    fillHt(ht,n,nxw,xw,nwy[ew],wy+off,NULL,cXW,NULL,0);
+    double cmi=cmiHt(ht,cXW,cWY+off,wy2w+off,cW+off);
+
     ms[ee]=(ms[ee]<cmi)?ms[ee]:cmi;
     //Maybe it is already eliminated?
     if(tbs>ms[ee]) break;
